import React, { createContext, useContext, useState, useCallback } from 'react';
import { Box, Alert, Button, Icon, Text } from '@chakra-ui/react';
import { AnimatePresence, motion } from 'framer-motion';
import { AiOutlineInfoCircle, AiFillWarning } from 'react-icons/ai';

const NotificationContext = createContext(null);

/**
 * 通知hook
 * @returns {null}
 */
export const useNotification = () => useContext(NotificationContext);

/**
 * 通知根组件
 * @param children
 * @returns {JSX.Element}
 * @constructor
 */
export const NotificationProvider = ({ children }) => {
  const [notifications, setNotifications] = useState([]);

  const addNotification = useCallback((notification) => {
    const id = Date.now() + Math.random();
    setNotifications((prev) => [...prev, { ...notification, id }]);

    setTimeout(() => {
      setNotifications((prev) => prev.filter((n) => n.id !== id));
    }, 3000);
  }, []);

  const notify = {
    info: ({ title, description, button }) =>
      addNotification({
        type: 'info',
        title,
        description,
        button,
      }),
    error: ({ title, description, button }) =>
      addNotification({
        type: 'error',
        title,
        description,
        button,
      }),
  };

  const MotionBox = motion(Box);

  return (
    <NotificationContext.Provider value={notify}>
      {children}
      <Box
        position={'fixed'}
        bottom={4}
        right={4}
        zIndex={9999}
        display={'flex'}
        flexDirection={'column'}
        alignItems={'flex-end'}
      >
        <AnimatePresence>
          {notifications.map((item) => (
            <MotionBox
              key={item.id}
              layout
              initial={{ opacity: 0, x: 200 }}
              animate={{ opacity: 1, x: 0 }}
              exit={{ opacity: 0, x: 200 }}
              transition={{ type: 'spring', stiffness: 500, damping: 30 }}
              mb={3}
              width={'320px'}
            >
              <Box
                bg={item.type === 'error' ? '' : ''}
                color={'white'}
                borderRadius={'xl'}
                px={5}
                py={4}
                boxShadow={'2xl'}
                border={'1px solid rgba(255, 255, 255, 0.4)'}
                backdropFilter={'blur(8px)'}
              >
                <Box display={'flex'} alignItems={'center'}>
                  <Icon
                    as={item.type === 'error' ? AiFillWarning : AiOutlineInfoCircle}
                    boxSize={6}
                    mr={3}
                  />
                  <Text fontSize={'lg'} fontWeight={'bold'}>
                    {item.title}
                  </Text>
                </Box>
                <Text mt={2}>{item.description}</Text>
                {item.button && (
                  <Button
                    colorPalette={'whiteAlpha'}
                    size={'sm'}
                    mt={3}
                    onClick={item.button.onClick}
                  >
                    {item.button.label}
                  </Button>
                )}
              </Box>
            </MotionBox>
          ))}
        </AnimatePresence>
      </Box>
    </NotificationContext.Provider>
  );
};
